@@ -32,7 +32,7 @@
 #include <ctype.h>
 #include <inttypes.h>
 
-#include <bluetooth/bluetooth.h>
+#include "lib/bluetooth.h"
 
 #include "src/shared/util.h"
 #include "bt.h"
@@ -90,13 +90,13 @@ struct rfcomm_lhdr {
 	uint16_t length;
 	uint8_t fcs;
 	uint8_t credits; /* only for UIH frame */
-} __attribute__((packed));
+};
 
 struct rfcomm_lmsc {
 	uint8_t dlci;
 	uint8_t v24_sig;
 	uint8_t break_sig;
-} __attribute__((packed));
+};
 
 struct rfcomm_rpn {
 	uint8_t dlci;
@@ -106,21 +106,21 @@ struct rfcomm_rpn {
 	uint8_t xon;
 	uint8_t xoff;
 	uint16_t pm;
-} __attribute__ ((packed));
+};
 
 struct rfcomm_rls {
 	uint8_t dlci;
 	uint8_t error;
-} __attribute__((packed));
+};
 
 struct rfcomm_nsc {
 	uint8_t cmd_type;
-} __attribute__((packed));
+};
 
 struct rfcomm_lmcc {
 	uint8_t type;
 	uint16_t length;
-} __attribute__((packed));
+};
 
 struct rfcomm_frame {
 	struct rfcomm_lhdr hdr;
@@ -265,6 +265,7 @@ static inline bool mcc_pn(struct rfcomm_frame *rfcomm_frame, uint8_t indent)
 {
 	struct l2cap_frame *frame = &rfcomm_frame->l2cap_frame;
 	struct rfcomm_pn pn;
+	uint16_t mtu;
 
 	/* rfcomm_pn struct is defined in rfcomm.h */
 
@@ -284,9 +285,11 @@ static inline bool mcc_pn(struct rfcomm_frame *rfcomm_frame, uint8_t indent)
 	if (!l2cap_frame_get_u8(frame, &pn.ack_timer))
 		return false;
 
-	if (!l2cap_frame_get_le16(frame, &pn.mtu))
+	if (!l2cap_frame_get_le16(frame, &mtu))
 		return false;
 
+	pn.mtu = mtu;
+
 	if (!l2cap_frame_get_u8(frame, &pn.max_retrans))
 		return false;
 
@@ -367,7 +370,7 @@ static inline bool mcc_frame(struct rfcomm_frame *rfcomm_frame, uint8_t indent)
 	else
 		type_str = "Unknown";
 
-	print_field("%*cMCC Message type: %s %s(0x%2.2x)", indent, ' ',
+	print_field("%*cMCC Message type: %s %s (0x%2.2x)", indent, ' ',
 				type_str, CR_STR(mcc.type), type);
 
 	print_field("%*cLength: %d", indent+2, ' ', mcc.length);
@@ -421,7 +424,7 @@ struct rfcomm_data {
 };
 
 static const struct rfcomm_data rfcomm_table[] = {
-	{ 0x2f, "Set Async Balance Mode (SABM) " },
+	{ 0x2f, "Set Async Balance Mode (SABM)" },
 	{ 0x63, "Unnumbered Ack (UA)" },
 	{ 0x0f, "Disconnect Mode (DM)" },
 	{ 0x43, "Disconnect (DISC)" },
@@ -493,7 +496,7 @@ void rfcomm_packet(const struct l2cap_frame *frame)
 	}
 
 	print_indent(6, frame_color, "RFCOMM: ", frame_str, COLOR_OFF,
-						"(0x%2.2x)", ctype);
+						" (0x%2.2x)", ctype);
 
 	rfcomm_frame.hdr = hdr;
 	print_rfcomm_hdr(&rfcomm_frame, indent);
