@@ -1018,6 +1018,9 @@ static const char *get_player_name(void *user_data)
 {
 	struct media_player *mp = user_data;
 
+	if (!mp->name)
+		return "Player";
+
 	return mp->name;
 }
 
