@@ -65,6 +65,7 @@ struct btd_gatt_client {
 
 	struct queue *services;
 	struct queue *all_notify_clients;
+	struct queue *ios;
 };
 
 struct service {
@@ -1086,6 +1087,8 @@ static void pipe_io_destroy(struct pipe_io *io)
 static void characteristic_destroy_pipe(struct characteristic *chrc,
 							struct io *io)
 {
+	queue_remove(chrc->service->client->ios, io);
+
 	if (chrc->write_io && io == chrc->write_io->io) {
 		pipe_io_destroy(chrc->write_io);
 		chrc->write_io = NULL;
@@ -1169,6 +1172,8 @@ static DBusMessage *characteristic_create_pipe(struct characteristic *chrc,
 						"NotifyAcquired");
 	}
 
+	queue_push_tail(chrc->service->client->ios, io);
+
 	DBG("%s: sender %s io %p", dbus_message_get_member(msg),
 					dbus_message_get_sender(msg), io);
 
@@ -1187,7 +1192,7 @@ static void characteristic_ready(bool success, uint8_t ecode, void *user_data)
 
 	chrc->ready_id = 0;
 
-	if (chrc->write_io->msg) {
+	if (chrc->write_io && chrc->write_io->msg) {
 		reply = characteristic_create_pipe(chrc, chrc->write_io->msg);
 
 		g_dbus_send_message(btd_get_dbus_connection(), reply);
@@ -1196,7 +1201,7 @@ static void characteristic_ready(bool success, uint8_t ecode, void *user_data)
 		chrc->write_io->msg = NULL;
 	}
 
-	if (chrc->notify_io->msg) {
+	if (chrc->notify_io && chrc->notify_io->msg) {
 		reply = characteristic_create_pipe(chrc, chrc->notify_io->msg);
 
 		g_dbus_send_message(btd_get_dbus_connection(), reply);
@@ -1426,23 +1431,31 @@ static void register_notify_io_cb(uint16_t att_ecode, void *user_data)
 {
 	struct notify_client *client = user_data;
 	struct characteristic *chrc = client->chrc;
+	struct bt_gatt_client *gatt = chrc->service->client->gatt;
 
-	if (!att_ecode)
+	if (att_ecode) {
+		queue_remove(chrc->notify_clients, client);
+		notify_client_free(client);
 		return;
+	}
 
-	queue_remove(chrc->notify_clients, client);
-	notify_client_free(client);
+	if (!bt_gatt_client_is_ready(gatt)) {
+		if (!chrc->ready_id)
+			chrc->ready_id = bt_gatt_client_ready_register(gatt,
+							characteristic_ready,
+							chrc, NULL);
+		return;
+	}
 
-	pipe_io_destroy(chrc->notify_io);
-	chrc->notify_io = NULL;
+	characteristic_ready(true, 0, chrc);
 }
 
 static void notify_io_destroy(void *data)
 {
 	struct notify_client *client = data;
 
-	queue_remove(client->chrc->notify_clients, client);
-	notify_client_unref(client);
+	if (queue_remove(client->chrc->notify_clients, client))
+		notify_client_unref(client);
 }
 
 static DBusMessage *characteristic_acquire_notify(DBusConnection *conn,
@@ -1482,18 +1495,10 @@ static DBusMessage *characteristic_acquire_notify(DBusConnection *conn,
 
 	chrc->notify_io = new0(struct pipe_io, 1);
 	chrc->notify_io->data = client;
+	chrc->notify_io->msg = dbus_message_ref(msg);
 	chrc->notify_io->destroy = notify_io_destroy;
 
-	if (!bt_gatt_client_is_ready(gatt)) {
-		if (!chrc->ready_id)
-			chrc->ready_id = bt_gatt_client_ready_register(gatt,
-							characteristic_ready,
-							chrc, NULL);
-		chrc->notify_io->msg = dbus_message_ref(msg);
-		return NULL;
-	}
-
-	return characteristic_create_pipe(chrc, msg);
+	return NULL;
 }
 
 static DBusMessage *characteristic_start_notify(DBusConnection *conn,
@@ -1581,8 +1586,7 @@ static DBusMessage *characteristic_stop_notify(DBusConnection *conn,
 		return btd_error_failed(msg, "No notify session started");
 
 	if (chrc->notify_io) {
-		pipe_io_destroy(chrc->notify_io);
-		chrc->notify_io = NULL;
+		characteristic_destroy_pipe(chrc, chrc->notify_io->io);
 		return dbus_message_new_method_return(msg);
 	}
 
@@ -1643,11 +1647,15 @@ static void characteristic_free(void *data)
 	queue_destroy(chrc->descs, NULL);
 	queue_destroy(chrc->notify_clients, NULL);
 
-	if (chrc->write_io)
+	if (chrc->write_io) {
+		queue_remove(chrc->service->client->ios, chrc->write_io->io);
 		pipe_io_destroy(chrc->write_io);
+	}
 
-	if (chrc->notify_io)
+	if (chrc->notify_io) {
+		queue_remove(chrc->service->client->ios, chrc->notify_io->io);
 		pipe_io_destroy(chrc->notify_io);
+	}
 
 	g_free(chrc->path);
 	free(chrc);
@@ -2063,6 +2071,7 @@ struct btd_gatt_client *btd_gatt_client_new(struct btd_device *device)
 	client = new0(struct btd_gatt_client, 1);
 	client->services = queue_new();
 	client->all_notify_clients = queue_new();
+	client->ios = queue_new();
 	client->device = device;
 	ba2str(device_get_address(device), client->devaddr);
 
@@ -2078,6 +2087,7 @@ void btd_gatt_client_destroy(struct btd_gatt_client *client)
 
 	queue_destroy(client->services, unregister_service);
 	queue_destroy(client->all_notify_clients, NULL);
+	queue_destroy(client->ios, NULL);
 	bt_gatt_client_unref(client->gatt);
 	gatt_db_unref(client->db);
 	free(client);
@@ -2199,6 +2209,9 @@ void btd_gatt_client_disconnected(struct btd_gatt_client *client)
 
 	DBG("Device disconnected. Cleaning up.");
 
+	queue_remove_all(client->ios, NULL, NULL,
+				(queue_destroy_func_t) io_shutdown);
+
 	/*
 	 * TODO: Once GATT over BR/EDR is properly supported, we should pass the
 	 * correct bdaddr_type based on the transport over which GATT is being
